@@ -17,11 +17,19 @@
 #' e.g., for 'ID=' use (file, specID = "ID") and for 'IMAGE=' use (file, specID = "imageID").
 #' The default is specID="None".
 #' 
-#' NOTE: At present, all other information that can be contained in tps files (curves, comments, variables, radii, etc.)
+#' If there are curves defined in the file (i.e., CURVES= fields), the option readcurves should be used.
+#' When readcurves = TRUE, the coordinate data for the curves will be returned as semilandmarks and will be appended to
+#' the fixed landmark data. Then the user needs to use \code{\link{define.sliders.2d}} or \code{\link{define.sliders.3d}}
+#' to create a matrix designating how the curve points will slide (used by 'curves=' in \code{\link{gpagen}}).
+#' When readcurves = FALSE, only the landmark data are returned.
+#' 
+#' NOTE: At present, all other information that can be contained in tps files (comments, variables, radii, etc.)
 #'   is ignored. 
 #'
 #' @param file A *.tps file containing two- or three-dimensional landmark data
 #' @param specID a character specifying whether to extract the specimen ID names from the ID or IMAGE lines (default is "None").
+#' @param readcurves A logical value stating whether CURVES= field and associated coordinate data will be read as semilandmarks (TRUE)
+#' or ignored (FALSE).
 #' @param warnmsg A logical value stating whether warnings should be printed
 #' @export
 #' @keywords IO
@@ -29,30 +37,33 @@
 #' @return Function returns a (p x k x n) array, where p is the number of landmark points, k is the number 
 #'   of landmark dimensions (2 or 3), and n is the number of specimens. The third dimension of this array 
 #'   contains names for each specimen, which are obtained from the image names in the *.tps file. 
+#'   
 #' @references  Rohlf, F. J. 2010. tpsRelw: Relative warps analysis. Version 1.49. Department of Ecology 
 #'   and Evolution, State University of New York at Stony Brook, Stony Brook, NY.
 
-readland.tps <- function (file, specID = c("None", "ID", "imageID"), warnmsg = T) 
+readland.tps <- function (file, specID = c("None", "ID", "imageID"), 
+                          readcurves = FALSE, warnmsg = T) 
 {
+  ignore.case = TRUE
   specID <- match.arg(specID)
   tpsfile <- scan(file = file, what = "char", sep = "\n", quiet = TRUE)
-  lmdata <- grep("LM=", tpsfile)
-  if (length(lmdata) == 0) {
-    lmdata <- grep("LM3=", tpsfile)
-    nland <- as.numeric(sub("LM3=", "", tpsfile[lmdata]))
-    k <- 3
-  }
-  else {
-    nland <- as.numeric(sub("LM=", "", tpsfile[lmdata]))
+  lmdata <- grep("LM=", tpsfile, ignore.case)
+  if (length(lmdata !=0)) {
+    nland <- as.numeric(sub("LM=", "", tpsfile[lmdata], ignore.case))
     k <- 2
   }
+  if (length(lmdata) == 0) {
+    lmdata <- grep("LM3=", tpsfile, ignore.case)
+    nland <- as.numeric(sub("LM3=", "", tpsfile[lmdata], ignore.case))
+    k <- 3
+  }
   n <- nspecs <- length(lmdata)
   if (max(nland) - min(nland) != 0) {
     stop("Number of landmarks not the same for all specimens.")
   }
   p <- nland[1]
   imscale <- as.numeric(sub("SCALE=", "", tpsfile[grep("SCALE", 
-                                                       tpsfile)]))
+                                                       tpsfile, ignore.case)], ignore.case))
   if (is.null(imscale)) {
     imscale = array(1, nspecs)
   }
@@ -64,10 +75,28 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"), warnmsg = T
   if (length(imscale) != nspecs) {
     imscale = array(1, nspecs)
   }
+  crvs <- grep("CURVES=", tpsfile, ignore.case)
+  if (readcurves == TRUE){ 
+    if (length(crvs) == 0){ stop("No CURVES= field present in file") } 
+    ncurve <- as.numeric(sub("CURVES=", "", tpsfile[crvs], ignore.case))
+    ncurvepts <- as.numeric(sub("POINTS=", "", tpsfile[grep("POINTS=", tpsfile, ignore.case)], ignore.case))
+      if (max(ncurve) - min(ncurve) != 0) {
+        stop("Number of curves not the same for all specimens.") }
+      if (warnmsg == T) {print(paste("Landmarks 1:", p, " are fixed landmarks.", sep=""))
+                         print(paste("Landmarks ", p+1, ":", p+sum(ncurvepts[1:ncurve[1]]), " are semilandmarks.", sep=""))}
+      p <- nland[1] + sum(ncurvepts[1:ncurve[1]]) 
+      }
+  if (readcurves == FALSE){ 
+    pts <- grep("POINTS=", tpsfile, ignore.case)
+    pts <- cbind(pts+1, (pts + pts))
+    outlinepts <- NULL
+    for(i in 1:nrow(pts)){ outlinepts <- c(outlinepts, (pts[i,1]:pts[i,2]))}
+    tpsfile <- tpsfile[-outlinepts]
+  }
   tmp <- tpsfile[-(grep("=", tpsfile))]
   options(warn = -1)
-  tmp <- matrix(as.numeric(unlist(strsplit(tmp, split = " +")), 
-                           ncol = k, byrow = T))
+  tmp <- matrix(as.numeric(unlist(strsplit(tmp, split = " +"))),ncol = k, byrow = T)
+ 
   if (warnmsg == T) {
     if (sum(which(is.na(tmp) == TRUE)) > 0) {
       print("NOTE.  Missing data identified.")
@@ -83,14 +112,15 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"), warnmsg = T
     }
   }
   if (specID == "imageID") {
-    imageID <- (sub("IMAGE=", "", tpsfile[grep("IMAGE", tpsfile)]))
+    imageID <- (sub("IMAGE=", "", tpsfile[grep("IMAGE", tpsfile, ignore.case)], 
+                    ignore.case))
     if (length(imageID) != 0) {
-      imageID <- sub(".jpg", "", imageID)
-      imageID <- sub(".tif", "", imageID)
-      imageID <- sub(".bmp", "", imageID)
-      imageID <- sub(".tiff", "", imageID)
-      imageID <- sub(".jpeg", "", imageID)
-      imageID <- sub(".jpe", "", imageID)
+      imageID <- sub(".jpg", "", imageID, ignore.case)
+      imageID <- sub(".tif", "", imageID, ignore.case)
+      imageID <- sub(".bmp", "", imageID, ignore.case)
+      imageID <- sub(".tiff", "", imageID, ignore.case)
+      imageID <- sub(".jpeg", "", imageID, ignore.case)
+      imageID <- sub(".jpe", "", imageID, ignore.case)
       dimnames(coords)[[3]] <- as.list(imageID)
       if (warnmsg == T) {
         print("Specimen names extracted from line IMAGE=")
@@ -104,7 +134,7 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"), warnmsg = T
   }
   
   if (specID == "ID") {
-    ID <- sub("ID=", "", tpsfile[grep("ID", tpsfile)])
+    ID <- sub("ID=", "", tpsfile[grep("ID", tpsfile, ignore.case)], ignore.case)
     if (length(ID) == 0) {
       if(warnmsg ==T){
         print("No name given under ID=. Specimen names not extracted")
@@ -117,5 +147,5 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"), warnmsg = T
       }
     }
   }
-  return(coords = coords)
+return(coords = coords)                    
 }
\ No newline at end of file
